<?php
/**
 * WooCommerce Jilt
 *
 * This source file is subject to the GNU General Public License v3.0
 * that is bundled with this package in the file license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.gnu.org/licenses/gpl-3.0.html
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@jilt.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade WooCommerce Jilt to newer
 * versions in the future. If you wish to customize WooCommerce Jilt for your
 * needs please refer to http://help.jilt.com/jilt-for-woocommerce
 *
 * @package   WC-Jilt/Admin
 * @author    Jilt
 * @category  Admin
 * @copyright Copyright (c) 2015-2018, SkyVerge, Inc.
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License v3.0
 */

defined( 'ABSPATH' ) or exit;

/**
 * Main integration class
 *
 * @since 1.0.0
 */
class WC_Jilt_Integration extends WC_Integration {


	/** @var \WC_Jilt_API instance */
	private $api;

	/** @var \WC_Jilt_OAuth_Access_Token the access token instance */
	private $access_token;

	/** @var string the API secret key */
	private $secret_key;

	/** @var \WC_Jilt_Integration_Admin instance */
	private $admin;


	/**
	 * Initialize the class
	 *
	 * @since 1.0.0
	 * @see WC_Jilt_Integration::instance()
	 */
	public function __construct() {

		// @see WC_Settings_API::$id
		$this->id = 'jilt';

		// delegate admin-related setup
		$this->admin = new WC_Jilt_Integration_Admin( $this );

		// load settings
		$this->init_settings();

		if ( $this->is_linked() ) {

			// handle placed orders and keeping the financial status in sync with Jilt
			add_action( 'woocommerce_order_status_changed', array( $this, 'order_status_changed' ), 10, 3 );
		}

		// load styles/scripts
		add_action( 'admin_enqueue_scripts', array( $this, 'load_styles_scripts' ) );
	}


	/**
	 * Loads admin styles and scripts.
	 *
	 * @internal
	 *
	 * @since 1.4.0
	 */
	public function load_styles_scripts() {

		$screen = get_current_screen();
		$load_styles = false;

		if ( wc_jilt()->is_plugin_settings() ) {

			wp_enqueue_script( 'jilt-for-woocommerce-admin', wc_jilt()->get_plugin_url() . '/assets/js/admin/wc-jilt-admin.min.js', array( 'jquery' ), WC_Jilt::VERSION );

			wp_localize_script( 'jilt-for-woocommerce-admin', 'wc_jilt', array(
				'ajax_url' => admin_url( 'admin-ajax.php' ),
				'i18n'     => array(
					'confirm_disconnect' => esc_html__( 'Are you sure you want to disconnect your shop from your Jilt account?', 'jilt-for-woocommerce' ),
				),
			) );

			$load_styles = true;
		}

		if ( 'plugin-install' === $screen->id || $load_styles ) {
			wp_enqueue_style( 'jilt-for-woocommerce-admin', wc_jilt()->get_plugin_url() . '/assets/css/admin/wc-jilt-admin.min.css', WC_Jilt::VERSION );
		}
	}


	/**
	 * Return the URL to the specified page within the Jilt web app, useful
	 * for direct linking to internal pages, like campaigns
	 *
	 * @since 1.0.0
	 * @param string $page page URL partial, e.g. 'dashboard'
	 * @return string
	 */
	public function get_jilt_app_url( $page = '' ) {
		return sprintf( 'https://' . $this->get_plugin()->get_app_hostname() . '/shops/%1$d/%2$s', (int) $this->get_linked_shop_id(), rawurlencode( $page ) );
	}


	/**
	 * Gets the plugin settings
	 *
	 * @see WC_Settings_API::settings
	 * @since 1.1.0
	 * @return array associative array of plugin settings including the following keys:
	 *   - 'secret_key': string
	 *   - 'log_level': 100...900
	 *   - 'recover_held_orders': 'yes'|'no'
	 */
	public function get_settings() {
		return $this->settings;
	}


	/**
	 * Updates the plugin settings.
	 *
	 * @since 1.1.0
	 *
	 * @param array $data associative array of settings
	 */
	public function update_settings( $data ) {
		update_option( $this->get_option_key(), $data );
	}


	/**
	 * Clears out the the Jilt connection data.
	 *
	 * This includes: access token, public key, shop id, current shop domain, is disabled.
	 *
	 * @since 1.1.0
	 */
	public function clear_connection_data() {

		// TODO: remove the two following lines when dropping support for secret key auth
		delete_option( 'wc_jilt_secret_key' );
		delete_option( 'wc_jilt_public_key' );

		delete_option( 'wc_jilt_shop_id' );
		delete_option( 'wc_jilt_shop_domain' );
		delete_option( 'wc_jilt_disabled' );

		$this->clear_access_token();

		delete_option( 'wc_jilt_client_id' );
		delete_option( 'wc_jilt_client_secret' );

		$this->api = null; // reset API instance
	}


	/**
	 * Returns the Jilt API instance.
	 *
	 * Since 1.4.0 this always returns an API instance, even if not authenticated.
	 *
	 * @since 1.0.0
	 *
	 * @return WC_Jilt_API the API instance
	 */
	public function get_api() {

		// override the current auth token with a new one?
		if ( null !== $this->api && $this->api->get_auth_token() !== $this->get_auth_token() ) {
			$this->api = null;
		}

		if ( null === $this->api ) {
			$this->set_api(
				new WC_Jilt_API(
					$this->get_linked_shop_id(),
					$this->get_auth_token()
				)
			);
		}

		return $this->api;
	}


	/**
	 * Checks the site URL to determine whether this is likely a duplicate site.
	 *
	 * The typical case is when a production site is copied to a staging server
	 * in which case all of the Jilt keys will be copied as well, and staging
	 * will happily make production API requests.
	 *
	 * The one false positive that can happen here is if the site legitimately
	 * changes domains. Not sure yet how you would handle this, might require
	 * some administrator intervention.
	 *
	 * @since 1.1.0
	 *
	 * @return boolean true if this is likely a duplicate site
	 */
	public function is_duplicate_site() {
		$shop_domain = $this->get_linked_shop_domain();

		return $shop_domain && $shop_domain !== $this->get_plugin()->get_shop_domain();
	}


	/**
	 * Returns the auth token for Jilt API - either OAuth access token or secret api key.
	 *
	 * @since 1.4.0
	 *
	 * @return WC_Jilt_OAuth_Access_Token|string|null OAuth access token or secret api key, or null if not available
	 */
	public function get_auth_token() {
		return 'secret_key' === $this->get_auth_method() ? $this->get_secret_key() : $this->get_access_token();
	}


	/**
	 * Returns the configured secret key.
	 *
	 * @since 1.0.0
	 *
	 * @return string the secret key, if set, null otherwise
	 */
	public function get_secret_key() {

		if ( null === $this->secret_key ) {
			// retrieve from db if not already set
			$this->set_secret_key( $this->get_option( 'secret_key' ) );
		}

		return $this->secret_key;
	}


	/**
	 * Sets the secret key.
	 *
	 * @since 1.2.0
	 *
	 * @param string $secret_key the secret key
	 */
	public function set_secret_key( $secret_key ) {

		$this->secret_key = $secret_key;
	}


	/**
	 * Checks whether the plugin configured.
	 *
	 * @since 1.0.0
	 *
	 * @return bool true if the plugin is configured, false otherwise
	 */
	public function is_configured() {
		// if we have an authentication token (either the legacy secret key or an oauth access token), we're good to go
		return (bool) $this->get_auth_token();
	}


	/**
	 * Checks whether the plugin has connected to Jilt.
	 *
	 * @since 1.0.0
	 *
	 * @return bool true if the plugin has connected to Jilt
	 */
	public function has_connected() {

		if ( 'secret_key' === $this->get_auth_method() ) {

			// since the public key is returned by the REST API it serves as a
			// reasonable proxy for whether we've connected with the current secret key
			// note that we get the option directly
			return (bool) get_option( 'wc_jilt_public_key' );
		}

		// since the oauth access token is saved only after the site is authorized,
		// we can use it to determine whether the site is connected to Jilt
		return (bool) $this->get_auth_token();
	}


	/**
	 * Checks whether this shop has linked itself to a Jilt account.
	 *
	 * @since 1.0.0
	 *
	 * @return boolean true if this shop is linked
	 */
	public function is_linked() {
		return (bool) $this->get_linked_shop_id();
	}


	/**
	 * Returns the linked Jilt shop identifier for this site, if any.
	 *
	 * @since 1.0.0
	 * @return int Jilt shop identifier, or null
	 */
	public function get_linked_shop_id() {
		return get_option( 'wc_jilt_shop_id', null );
	}


	/**
	 * Persists the given linked shop identifier.
	 *
	 * @since 1.0.0
	 *
	 * @param int $id the linked Shop identifier
	 * @return int the provided $id
	 */
	public function set_linked_shop_id( $id ) {

		update_option( 'wc_jilt_shop_id', $id );

		if ( 'secret_key' === $this->get_auth_method() ) {
			$this->stash_secret_key( $this->get_secret_key() );
		}

		// clear the API object so that the new shop id can be used for subsequent requests
		if ( null !== $this->api && $this->api->get_shop_id() !== $id ) {
			$this->api->set_shop_id( $id );
		}

		return $id;
	}


	/**
	 * Checks whether the integration is disabled.
	 *
	 * When disabled, this indicates that although the plugin is
	 * installed, activated, and configured, it should not send any requests
	 * over the Jilt REST API.
	 *
	 * Since 1.4.0 this simply indicates that the site is detected to be duplicated (e.g.
	 * a production site that was migrated to staging).
	 *
	 * @since 1.1.0
	 *
	 * @return bool
	 */
	public function is_disabled() {
		return $this->is_duplicate_site();
	}


	/**
	 * Checks whether the shop is connected to Jilt and active or not.
	 *
	 * @since 1.4.0
	 *
	 * @return bool
	 */
	public function is_jilt_connected() {
		return $this->is_configured() && $this->is_linked() && ! $this->is_disabled();
	}


	/**
	 * Get the secret key stash
	 *
	 * @since 1.2.0
	 * @return array of secret key strings
	 */
	public function get_secret_key_stash() {
		$stash = get_option( 'wc_jilt_secret_key_stash', array() );

		if ( ! is_array( $stash ) ) {
			$stash = array();
		}

		return $stash;
	}


	/**
	 * Stashes the current secret key into the db.
	 *
	 * @since 1.1.0
	 *
	 * @param string $secret_key the secret key to stash
	 */
	public function stash_secret_key( $secret_key ) {

		// What is the purpose of all this you might ask? Well it provides us a
		// future means of validating/handling recovery URLs that were generated
		// with a prior secret key
		$stash = $this->get_secret_key_stash();

		if ( ! in_array( $secret_key, $stash, true ) ) {
			$stash[] = $secret_key;
		}

		update_option( 'wc_jilt_secret_key_stash', $stash );
	}


	/**
	 * Sets the access token.
	 *
	 * @since 1.4.0
	 *
	 * @param array $token oauth access token
	 */
	public function set_access_token( $token ) {

		update_option( 'wc_jilt_access_token', $token );

		$this->init_access_token( $token );
	}


	/**
	 * Returns the access token.
	 *
	 * @since 1.4.0
	 *
	 * @return \WC_Jilt_OAuth_Access_token|null jilt access token instance or null if not available
	 */
	public function get_access_token() {

		if ( ! isset( $this->access_token ) ) {
			$this->init_access_token( get_option( 'wc_jilt_access_token' ) );
		}

		return $this->access_token ? $this->access_token : null;
	}


	/**
	 * Initializes the access token.
	 *
	 * @since 1.4.0
	 *
	 * @param array $token token args
	 */
	private function init_access_token( $token ) {
		$this->access_token = is_array( $token ) ? new WC_Jilt_OAuth_Access_token( $token ) : null;
	}


	/**
	 * Clears the access token.
	 *
	 * @since 1.4.0
	 */
	public function clear_access_token() {

		delete_option( 'wc_jilt_access_token' );

		$this->access_token = null;
	}


	/**
	 * Sets the shop public key.
	 *
	 * @since 1.4.0
	 *
	 * @param string $key shop public key
	 */
	public function set_public_key( $key ) {
		update_option( 'wc_jilt_public_key', $key );
	}


	/**
	 * Returns the authentication method used for REST API calls.
	 *
	 * Secret key authentication is deprecated since 1.4.0 and is used only to provide
	 * backwards compatibility for shops that haven't upgraded to OAuth2 access token yet.
	 *
	 * @since 1.4.0
	 *
	 * @return string the authentication method, either 'secret_key' or 'access_token'
	 */
	public function get_auth_method() {
		return $this->get_secret_key() ? 'secret_key' : 'access_token';
	}


	/**
	 * Persists the given linked Shop identifier.
	 *
	 * @since 1.1.0
	 *
	 * @return String the shop domain that was set
	 */
	public function set_shop_domain() {

		_deprecated_function( 'WC_Jilt_Integration::set_shop_domain()', '1.4.0', 'WC_Jilt_Integration::set_linked_shop_domain' );

		return $this->set_linked_shop_domain();
	}


	/**
	 * Persists the linked shop domain for historical reference.
	 *
	 * @since 1.4.0
	 *
	 * @return string the shop domain that was set
	 */
	public function set_linked_shop_domain() {

		$shop_domain = $this->get_plugin()->get_shop_domain();

		// prevent migration plugins from overriding the domain by masking it, so we can
		// detect later if the site has been moved and act accordingly
		$shop_domain = str_replace( '.', '[.]', $shop_domain );

		update_option( 'wc_jilt_shop_domain', $shop_domain );

		return $shop_domain;
	}


	/**
	 * Returns the stored shop domain.
	 *
	 * @since 1.4.0
	 *
	 * @return string the shop domain that was stored when connecting to Jilt
	 */
	public function get_linked_shop_domain() {
		return str_replace( '[.]', '.', get_option( 'wc_jilt_shop_domain', '' ) );
	}


	/**
	 * Checks whether held orders should be considered as placed.
	 *
	 * @since 1.1.0
	 *
	 * @return boolean true if "on-hold" orders should not be considered as placed, false otherwise if "on-hold" should be considered recoverable
	 */
	public function recover_held_orders() {

		// when updating settings, make sure we have the new value
		if ( isset( $_POST['woocommerce_jilt_recover_held_orders'] ) ) {
			return 'yes' === $_POST['woocommerce_jilt_recover_held_orders'];
		}

		return 'yes' === $this->get_option( 'recover_held_orders' );
	}


	/**
	 * Checks if the post-checkout registration prompt is enabled.
	 *
	 * @since 1.3.0
	 *
	 * @return boolean true if one-click registration is enabled
	 */
	public function allow_post_checkout_registration() {

		// when updating settings, make sure we have the new value
		if ( isset( $_POST['woocommerce_jilt_post_checkout_registration'] ) ) {
			return 'yes' === $_POST['woocommerce_jilt_post_checkout_registration'];
		}

		return 'yes' === $this->get_option( 'post_checkout_registration' );
	}


	/**
	 * Checks if the add-to-cart email prompt is enabled.
	 *
	 * @since 1.4.0
	 *
	 * @return boolean true if capture email on add-to-cart is enabled
	 */
	public function capture_email_on_add_to_cart() {

		// when updating settings, make sure we have the new value
		if ( isset( $_POST['woocommerce_jilt_capture_email_on_add_to_cart'] ) ) {
			return 'yes' === $_POST['woocommerce_jilt_capture_email_on_add_to_cart'];
		}

		return 'yes' === $this->get_option( 'capture_email_on_add_to_cart' );
	}


	/**
	 * Get base data for creating/updating a linked shop in Jilt
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function get_shop_data() {

		$theme = wp_get_theme();

		$data = array(
			'domain'                  => $this->get_plugin()->get_shop_domain(),
			'admin_url'               => admin_url(),
			'wordpress_site_url'      => get_home_url(),
			'profile_type'            => 'woocommerce',
			'woocommerce_version'     => WC()->version,
			'wordpress_version'       => get_bloginfo( 'version' ),
			'integration_version'     => $this->get_plugin()->get_version(),
			'php_version'             => PHP_VERSION,
			'name'                    => html_entity_decode( get_bloginfo( 'name' ) ),
			'main_theme'              => $theme->name,
			'currency'                => get_woocommerce_currency(),
			'primary_locale'          => strtolower( get_locale() ),
			'timezone'                => $this->get_store_timezone(),
			'created_at'              => $this->get_plugin()->get_wc_created_at(),
			'coupons_enabled'         => wc_coupons_enabled(),
			'free_shipping_available' => $this->is_free_shipping_available(),
			'integration_enabled'     => $this->is_linked() && ! $this->is_disabled(),
			'taxes_included'          => 'incl' === get_option( 'woocommerce_tax_display_cart' ),
		);

		// add shop address
		$data = array_merge(
			$data,
			$this->get_shop_address()
		);

		// avoid sending false negatives
		if ( $this->is_ssl() ) {
			$data['supports_ssl'] = true;
		}

		/**
		 * Filter shop data params used for updating the remote shop record via
		 * the API
		 *
		 * @since 1.3.2
		 *
		 * @param array $data the shop data
		 * @param WC_Jilt_Integration $this
		 */
		$data = apply_filters( 'wc_jilt_shop_data', $data, $this );

		return $data;
	}


	/** API methods ******************************************************/


	/**
	 * Link this shop to Jilt. The basic algorithm is to first attempt to
	 * create the shop over the Jilt API. If this request fails with a
	 * "Domain has already been taken" error, we try to find it over the Jilt
	 * API by domain, and update with the latest shop data.
	 *
	 * TODO: remove this when dropping secret key auth support {IT 2018-01-22}
	 *
	 * @since 1.0.0
	 *
	 * @return int the Jilt linked shop id, or false if the linking failed
	 * @throws SV_WC_API_Exception network exception or API error
	 */
	public function link_shop() {

		if ( $this->is_configured() && ! $this->is_duplicate_site() ) {

			$args = $this->get_shop_data();

			// set shop owner/email
			$current_user       = wp_get_current_user();
			$args['shop_owner'] = $current_user->user_firstname . ' ' . $current_user->user_lastname;
			$args['email']      = $current_user->user_email;

			try {

				$shop = $this->get_api()->create_shop( $args );
				$this->set_shop_domain();

				return $this->set_linked_shop_id( $shop->id );

			} catch ( SV_WC_API_Exception $exception ) {

				if ( SV_WC_Helper::str_exists( $exception->getMessage(), 'Domain has already been taken' ) ) {

					// log the exception and continue attempting to recover
					$this->get_plugin()->get_logger()->error( "Error communicating with Jilt: {$exception->getMessage()}" );

				} else {

					// for any error other than "Domain has already been taken" rethrow so the calling code can handle
					throw $exception;
				}
			}

			// if we're down here, it means that our attempt to create the
			// shop failed with "domain has already been taken". Lets try to
			// recover gracefully by finding the shop over the API
			$shop = $this->get_api()->find_shop( array( 'domain' => $args['domain'] ) );

			// no shop found? it might even exist, but the current API user might not have access to it
			if ( ! $shop ) {
				return false;
			}

			// we successfully found our shop. attempt to update it and save the ID
			try {

				// update the linked shop record with the latest settings
				$this->get_api()->update_shop( $args, $shop->id );

			} catch ( SV_WC_API_Exception $exception ) {

				// otherwise, log the exception
				$this->get_plugin()->get_logger()->error( "Error communicating with Jilt: {$exception->getMessage()}" );
			}

			$this->set_shop_domain();

			return $this->set_linked_shop_id( $shop->id );
		}
	}


	/**
	 * Unlink shop from Jilt
	 *
	 * @since 1.1.0
	 */
	public function unlink_shop() {

		// there is no remote Jilt shop for a duplicate site
		if ( $this->is_duplicate_site() ) {
			return;
		}

		try {
			// if the plugin is not configured properly (expired token, unable to refresh, no legacy secret key) we cannot unlink the shop
			if ( $this->is_configured() ) {
				$this->get_api()->delete_shop();
			}
		} catch ( SV_WC_API_Exception $exception ) {
			// quietly log any exception
			$this->get_plugin()->get_logger()->error( "Error communicating with Jilt when unlinking shop: {$exception->getMessage()}" );
		}
	}


	/**
	 * Revokes the integration plugin authorization.
	 *
	 * @since 1.4.0
	 */
	public function revoke_authorization() {

		try {
			// if the plugin is not configured properly (expired token, unable to refresh, no legacy secret key) we cannot revoke the token
			if ( $this->is_configured() ) {
				$this->get_api()->revoke_oauth_token( $this->get_client_id(), $this->get_client_secret() );
			}
		} catch ( SV_WC_API_Exception $exception ) {
			// quietly log any exception
			$this->get_plugin()->get_logger()->error( "Error communicating with Jilt when revoking OAuth token: {$exception->getMessage()}" );
		}
	}


	/**
	 * Update the shop info in Jilt once per day, useful for keeping track
	 * of which WP/WC versions are in use
	 *
	 * @since 1.0.0
	 * @throws SV_WC_API_Exception
	 */
	public function update_shop() {

		if ( ! $this->is_linked() || $this->is_duplicate_site() ) {
			return;
		}

		try {

			// update the linked shop record with the latest settings
			$this->get_api()->update_shop( $this->get_shop_data() );

		} catch ( SV_WC_API_Exception $exception ) {

			// disconnect if the shop isn't found (e.g. remote private key was changed)
			if ( $exception->getCode() == 404 ) {
				$this->clear_connection_data();
			}

			// log and rethrow the exception
			$this->get_plugin()->get_logger()->error( "Error communicating with Jilt: {$exception->getMessage()}" );

			if ( ! defined( 'DOING_CRON' ) ) {
				throw $exception;
			}
		}
	}


	/**
	 * Get and persist the public key for the current API user from the Jilt REST
	 * API
	 *
	 * @since 1.0.0
	 * @return string the public key
	 * @throws SV_WC_API_Exception on network exception or API error
	 */
	public function refresh_public_key() {

		return $this->get_public_key( true );
	}


	/**
	 * Returns the configured public key.
	 *
	 * Passing true will refresh the key from the Jilt REST API.
	 *
	 * @since 1.0.0
	 *
	 * @param boolean $refresh true if the current API user public key should be fetched from the Jilt API
	 * @return string the public key, if set
	 * @throws SV_WC_API_Exception on network exception or API error
	 */
	public function get_public_key( $refresh = false ) {

		$public_key = get_option( 'wc_jilt_public_key', null );

		if ( ( $refresh || ! $public_key ) && $this->is_configured() ) {
			update_option( 'wc_jilt_public_key', $this->get_api()->get_public_key() );
		}

		return $public_key;
	}


	/**
	 * Checks whether we have OAuth client credentials.
	 *
	 * @since 1.4.0
	 *
	 * @return bool true if we have credentials, false otherwise
	 */
	public function has_client_credentials() {
		return $this->get_client_id() && $this->get_client_secret();
	}


	/**
	 * Returns the OAuth client id.
	 *
	 * @since 1.4.0
	 *
	 * @return string
	 */
	public function get_client_id() {
		return get_option( 'wc_jilt_client_id' );
	}


	/**
	 * Returns the OAuth client secret.
	 *
	 * @since 1.4.0
	 *
	 * @return string
	 */
	public function get_client_secret() {
		return get_option( 'wc_jilt_client_secret' );
	}


	/**
	 * Sets the OAuth client secret.
	 *
	 * Also adds to the secret key stash.
	 *
	 * @since 1.4.1
	 *
	 * @param string $secret client secret key
	 * @return string
	 */
	public function set_client_secret( $secret ) {

		update_option( 'wc_jilt_client_secret', $secret );

		$this->stash_secret_key( $secret );
	}


	/** Other methods ******************************************************/


	/**
	 * Update related Jilt order when order status changes
	 *
	 * @since 1.0.0
	 * @param int $order_id order ID
	 * @param string $old_status
	 * @param string $new_status
	 */
	public function order_status_changed( $order_id, $old_status, $new_status ) {

		if ( $this->is_disabled() ) {
			return;
		}

		$cart_token = get_post_meta( $order_id, '_wc_jilt_cart_token', true );

		// bail out if this order does not have a cart token
		if ( ! $cart_token ) {
			return;
		}

		$order = new WC_Jilt_Order( $order_id );

		$jilt_placed_at    = $order->get_jilt_placed_at();
		$jilt_cancelled_at = $order->get_jilt_cancelled_at();

		// when a non-placed order transitions to a paid (processing/completed)
		// or on-hold status (unless "Recover Held Orders" is enabled), mark it
		// as placed. see also WC_Abstract_Order::update_status()
		if ( ! $jilt_placed_at && $this->is_placed( $order, $old_status, $new_status ) ) {

			$jilt_placed_at = current_time( 'timestamp', true );
			update_post_meta( $order_id, '_wc_jilt_placed_at', $jilt_placed_at );
			$jilt_placed_at = date( 'Y-m-d\TH:i:s\Z', $jilt_placed_at );
		}

		// handle order cancellation: when the status change is from pending ->
		// cancelled, that indicates WooCommerce auto-cancelling an unpaid order
		// from an off-site gateway, and in which case we'd want to consider the
		// order as recoverable. Any other status change to "cancelled" indicates
		// a user or admin cancellation
		if ( ! $jilt_cancelled_at && $old_status != 'pending' && 'cancelled' == $new_status ) {

			$jilt_cancelled_at = current_time( 'timestamp', true );
			update_post_meta( $order_id, '_wc_jilt_cancelled_at', $jilt_cancelled_at );
			$jilt_cancelled_at = date( 'Y-m-d\TH:i:s\Z', $jilt_cancelled_at );
		}

		$params = $order->get_jilt_order_data();

		$params['status'] = $new_status;

		if ( $jilt_placed_at ) {
			$params['placed_at'] = $jilt_placed_at;
		}
		if ( $jilt_cancelled_at ) {
			$params['cancelled_at'] = $jilt_cancelled_at;
		}

		// update Jilt order details
		try {

			$this->get_api()->update_order( $cart_token, $params );

		} catch ( SV_WC_API_Exception $exception ) {

			$this->get_plugin()->get_logger()->error( "Error communicating with Jilt: {$exception->getMessage()}" );
		}
	}


	/** Helper methods ******************************************************/


	/**
	 * Is the given order considered to be placed
	 *
	 * @since 1.4.0
	 * @param WC_Jilt_Order $order the order
	 * @param string $old_status
	 * @param string $new_status
	 * @return boolean true if $order is considered to be placed
	 */
	private function is_placed( $order, $old_status, $new_status ) {

		$placed = $order->is_paid() || ( $new_status == 'on-hold' && ! $this->recover_held_orders() );

		/**
		 * Filter whether the given $order is considered placed
		 *
		 * @since 1.4.0
		 * @param boolean $placed whether the order is considered placed
		 * @param WC_Jilt_Order $order the order
		 * @param string $old_status
		 * @param string $new_status
		 * @param WC_Jilt_Integration $this
		 */
		$placed = apply_filters( 'wc_jilt_order_is_placed', $placed, $order, $old_status, $new_status, $this );

		return $placed;
	}


	/**
	 * Return the timezone string for a store
	 *
	 * @since 1.2.0
	 * @return string
	 */
	protected function get_store_timezone() {
		return wc_timezone_string();
	}


	/**
	 * Is the current request being performed over ssl?
	 *
	 * This implementation does not use the wc_site_is_https() approach of
	 * testing the "home" wp option for "https" because that has been found not
	 * to be a very reliable indicator of SSL support.
	 *
	 * @since 1.2.0
	 * @return boolean true if the site is configured to use HTTPS
	 */
	protected function is_ssl() {
		return is_ssl();
	}


	/**
	 * Set the API object
	 *
	 * @since 1.1.0
	 * @param WC_Jilt_API $api the Jilt API object
	 */
	protected function set_api( $api ) {
		$this->api = $api;
	}


	/**
	 * Get the main plugin instance
	 *
	 * @since 1.2.0
	 * @return \WC_Jilt
	 */
	protected function get_plugin() {
		return wc_jilt();
	}


	/**
	 * Does there seem to be a coupon-enabled WC free shipping method available?
	 *
	 * Supports WC >= 2.6
	 *
	 * @return boolean true if there appears to be a coupon-enabled WC free
	 *   shipping method available.
	 */
	private function is_free_shipping_available() {
		global $wpdb;

		$zone_methods = $wpdb->get_results( "SELECT instance_id FROM {$wpdb->prefix}woocommerce_shipping_zone_methods as methods WHERE methods.method_id = 'free_shipping' AND is_enabled = 1" );
		foreach ( $zone_methods as $zone_method ) {
			$free_shipping_method = new WC_Shipping_Free_Shipping( $zone_method->instance_id );
			if ( in_array( $free_shipping_method->requires, array( 'coupon', 'either', 'both' ), true ) ) {
				return true;
			}
		}

		// legacy method enabled?
		// TODO: this can be removed when WC 3.0+ is required
		$legacy_free_shipping = get_option( 'woocommerce_free_shipping_settings' );
		if ( isset( $legacy_free_shipping['enabled'] ) &&
		     'yes' === $legacy_free_shipping['enabled'] &&
		     in_array( $legacy_free_shipping['requires'], array( 'coupon', 'either', 'both' ), true ) ) {
			return true;
		}

		return false;
	}


	/**
	 * Returns the shop address, including province_code/country_code for
	 * WC < 3.2.0, and address1, address2, city, and zip otherwise.
	 *
	 * @since 1.3.0
	 *
	 * @return array of shop address fields
	 */
	private function get_shop_address() {

		$base_location = wc_get_base_location();

		// state/country are always available
		$address = array(
			'province_code' => $base_location['state'],
			'country_code'  => $base_location['country'],
		);

		$wc_version = defined( 'WC_VERSION' ) && WC_VERSION ? WC_VERSION : null;

		// address1/address2/city/zip are available for later versions of WC
		if ( $wc_version && version_compare( $wc_version, '3.2.0', '>=' ) ) {
			$address = array_merge(
				$address,
				array(
					'address1' => WC()->countries->get_base_address(),
					'address2' => WC()->countries->get_base_address_2(),
					'city'     => WC()->countries->get_base_city(),
					'zip'      => WC()->countries->get_base_postcode(),
				)
			);
		}

		return $address;
	}


	/** Admin delegator methods ******************************************************/


	/**
	 * Initializes form fields in the format required by WC_Integration
	 *
	 * @see WC_Settings_API::init_form_fields()
	 * @since 1.0.0
	 */
	public function init_form_fields() {
		// delegate to admin instance
		$this->admin->init_form_fields();
	}


	/**
	 * Returns the Jilt Connection Status "setting" HTML fragment
	 *
	 * @see WC_Jilt_Integration_Admin::generate_jilt_status_html()
	 *
	 * @since 1.2.0
	 *
	 * @param mixed $key
	 * @param mixed $data
	 * @return string HTML fragment
	 */
	public function generate_jilt_status_html( $key, $data ) {
		return $this->admin->generate_jilt_status_html( $key, $data );
	}


}
