<?php
/**
 * WooCommerce Jilt
 *
 * This source file is subject to the GNU General Public License v3.0
 * that is bundled with this package in the file license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.gnu.org/licenses/gpl-3.0.html
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@jilt.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade WooCommerce Jilt to newer
 * versions in the future. If you wish to customize WooCommerce Jilt for your
 * needs please refer to http://help.jilt.com/jilt-for-woocommerce
 *
 * @package   WC-Jilt/Cart
 * @author    Jilt
 * @category  Frontend
 * @copyright Copyright (c) 2015-2018, SkyVerge, Inc.
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License v3.0
 */

defined( 'ABSPATH' ) or exit;

/**
 * Cart class
 *
 * Handles cart interactions
 *
 * @since 1.0.0
 */
class WC_Jilt_Cart_Handler {


	/**
	 * Setup class
	 *
	 * @since 1.0.0
	 */
	public function __construct() {

		add_action( 'woocommerce_cart_loaded_from_session', array( $this, 'handle_persistent_cart' ), 5 );

		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

		add_action( 'woocommerce_cart_updated', array( $this, 'cart_updated' ) );

		add_action( 'wp_login', array( $this, 'cart_updated' ) );

		$this->init();
	}


	/**
	 * Add hooks
	 *
	 * @since 1.0.0
	 */
	protected function init() {

		if ( $this->should_capture_email_on_add_to_cart() ) {
			add_action( 'wp_footer', array( $this, 'render_add_to_cart_popover' ) );
		}
	}


	/**
	 * Enqueues frontend scripts and styles.
	 *
	 * @since 1.0.0
	 */
	public function enqueue_scripts() {

		if ( wc_jilt()->get_integration()->is_disabled() ) {
			return;
		}

		if ( $this->should_capture_email_on_add_to_cart() ) {
			wp_enqueue_style( 'wc-jilt', wc_jilt()->get_plugin_url() . '/assets/css/frontend/wc-jilt-frontend.min.css', array(), wc_jilt()->get_version() );
		}

		// only load javascript once
		if ( wp_script_is( 'wc-jilt', 'enqueued' ) ) {
			return;
		}

		wp_enqueue_script( 'wc-jilt', wc_jilt()->get_plugin_url() . '/assets/js/frontend/wc-jilt-frontend.min.js', array(), wc_jilt()->get_version(), true );

		// script data
		$params = array(
			'public_key'            => wc_jilt()->get_integration()->get_public_key(),
			'order_address_mapping' => WC_Jilt_Order::get_jilt_order_address_mapping(),
			'orders_endpoint'       => $this->get_api()->get_orders_endpoint(),
			'cart_token'            => WC_Jilt_Session::get_cart_token(),
			'ajax_url'              => WC_AJAX::get_endpoint( '%%endpoint%%' ),
			'nonce'                 => wp_create_nonce( 'jilt-for-wc' ),
			'log_threshold'         => wc_jilt()->get_logger()->get_threshold(),
			'x_jilt_shop_domain'    => wc_jilt()->get_shop_domain(),
		);

		if ( $this->should_capture_email_on_add_to_cart() ) {

			wp_enqueue_script( 'jquery-webui-popover', 'https://cdn.jsdelivr.net/jquery.webui-popover/1.2.17/jquery.webui-popover.min.js', array(
				'jquery',
				'jquery-ui-core'
			), '1.2.1', false );
			wp_enqueue_style( 'jquery-webui-popover', 'https://cdn.jsdelivr.net/jquery.webui-popover/1.2.17/jquery.webui-popover.min.css', array(), '1.2.17' );
			wp_enqueue_script( 'jquery-typewatch', wc_jilt()->get_plugin_url() . '/assets/jquery.typewatch.min.js', array( 'jquery' ), '3.0.0' );

			$params['capture_email_on_add_to_cart'] = true;

			/**
			 * Filters the popover title for the add to cart email capture form.
			 *
			 * @since 1.4.0
			 *
			 * @param string $title the popover title
			 */
			$params['add_to_cart_title'] = apply_filters( 'wc_jilt_add_to_cart_popover_title', __( 'Reserve this item in your cart!', 'jilt-for-woocommerce' ) );

		}

		wp_localize_script( 'wc-jilt', 'wc_jilt_params', $params );
	}


	/**
	 * Handle loading/setting Jilt data for the persistent cart. This is important
	 * for two key situations:
	 *
	 * 1) when a user logs in and their persistent cart is loaded, the cart token
	 * and Jilt order ID is set in the session. This most commonly occurs when
	 * the customer visits the recovery URL and is logged in and their persistent
	 * cart loaded. If not done properly, this can result in duplicate Jilt orders.
	 * Note this is only done if the cart token exists in user meta AND there is
	 * no existing cart token in the session.
	 *
	 * 2) when a guest user (with an existing cart/session) logs in (usually on
	 * the checkout page), the cart token and Jilt order ID is saved to user meta,
	 * at roughly the same time as the persistent cart. This ensures that if the user
	 * leaves and logs back in days later, their persistent cart will be loaded along
	 * with the correct Jilt order that it was originally associated with. Note this
	 * is done only if the cart token exists in the session but NOT in user meta.
	 *
	 * This method is hooked into woocommerce_cart_loaded_from_session,
	 * which runs prior to the woocommerce_cart_updated action, which is where
	 * we hook in below to handle creating/updating Jilt orders.
	 *
	 * @since 1.0.0
	 */
	public function handle_persistent_cart() {

		// bail for guest users, when the cart is empty, or when doing a WP cron request
		if ( ! is_user_logged_in() || WC()->cart->is_empty() || defined( 'DOING_CRON' ) ) {
			return;
		}

		$user_id    = get_current_user_id();
		$cart_token = get_user_meta( $user_id, '_wc_jilt_cart_token', true );

		if ( $cart_token && ! WC_Jilt_Session::get_cart_token() ) {

			// for a logged in user with a persistent cart, set the cart token to the session
			WC_Jilt_Session::set_jilt_order_data( $cart_token );

		} elseif ( ! $cart_token && WC_Jilt_Session::get_cart_token() ) {

			// when a guest user with an existing cart logs in, save the cart token to user meta
			update_user_meta( $user_id, '_wc_jilt_cart_token', WC_Jilt_Session::get_cart_token() );
		}

		// persist order notes into the session during a pending recovery
		if ( WC_Jilt_Session::is_pending_recovery() && $order_note = get_user_meta( $user_id, '_wc_jilt_order_note', true ) ) {
			WC()->session->set( 'wc_jilt_order_note', $order_note );
			delete_user_meta( $user_id, '_wc_jilt_order_note' );
		}
	}


	/**
	 * Renders the popover HTML for collecting an email address when products
	 * are added to the cart.
	 *
	 * @internal
	 *
	 * @since 1.4.0
	 */
	public function render_add_to_cart_popover() {

		/**
		 * Filters the "enter it later" link text for the add to cart email capture form.
		 *
		 * @since 1.4.0
		 *
		 * @param string $dismiss_text
		 */
		$dismiss_text = apply_filters( 'wc_jilt_add_to_cart_popover_dismiss_text', __( "No thanks, I'll enter my email later.", 'jilt-for-woocommerce' ) );

		/**
		 * Filters the placeholder button label for the add to cart email capture form.
		 *
		 * @since 1.4.0
		 *
		 * @param string $label the button label
		 */
		$placeholder = apply_filters( 'wc_jilt_add_to_cart_popover_placeholder', __( 'Enter your email', 'jilt-for-woocommerce' ) );

		?>
		<div style="display: none;" id="wc-jilt-popover-content">
			<input name="wc-jilt-popover-email" class="js-wc-jilt-popover-email" type="email" placeholder="<?php echo esc_html( $placeholder ); ?>"/>
			<a href="#" class="js-wc-jilt-popover-bypass"><?php echo esc_html( $dismiss_text ); ?></a>
		</div>
		<?php
	}


	/** Event handlers ******************************************************/


	/**
	 * Create or update a Jilt order when cart is updated
	 *
	 * This is called at the bottom of WC_Checkout::set_session(), after the
	 * session and optional persistent cart are set.
	 *
	 * This will be called:
	 *
	 * + When a user signs into their account from the Checkout page (2x, once for /checkout/ and again for /wc-ajax/update_order_review/)
	 * + When a product is added to cart via ajax (1x)
	 * + When a product is added to cart via form submit (1x)
	 * + When a product is removed from the cart via 'x' link in widget (1x)
	 * + When a product is removed from the cart via 'x' link in cart (2x, once for /cart/?remove_item=6974ce5ac660610b44d9b9fed0ff9548 and then /cart/?removed_item=1)
	 * + When the cart page is loaded (1x)
	 * + When the checkout page is loaded (1x)
	 * + When the checkout form is submitted (I think, /wp-admin/admin-ajax.php?action=woocommerce_checkout)
	 * + When the address form on the checkout page is refreshed (via /wc-ajax/update_order_review/)
	 * + When a user logs into WordPress, either from the /wp-login.php page or on the WooCommerce my account page
	 *
	 * This will not be called:
	 *
	 * + When a customer creates an account from the checkout page
	 * + On the pay page
	 *
	 * @since 1.0.0
	 */
	public function cart_updated() {

		if ( wc_jilt()->get_integration()->is_disabled() ) {
			return;
		}

		if ( WC()->cart->is_empty() ) {
			// the cart can be temporarily empty for a variety of unknown reasons,
			// see https://github.com/skyverge/jilt-for-woocommerce/issues/102
			// return $this->cart_emptied();
			return;
		}

		// prevent duplicate updates when changing item quantities in the cart
		if ( isset( $_POST['cart'] ) ) {
			return;
		}

		$cart_token = WC_Jilt_Session::get_cart_token();

		if ( $cart_token ) {

			try {

				// update the existing Jilt order
				$this->get_api()->update_order( $cart_token, $this->get_cart_data() );

			} catch ( SV_WC_API_Exception $exception ) {

				// clear session so a new Jilt order can be created
				if ( 404 == $exception->getCode() ) {
					WC_Jilt_Session::unset_jilt_order_data();
					// try to create the order below
					$cart_token = null;
				}

				wc_jilt()->get_logger()->error( "Error communicating with Jilt: {$exception->getMessage()}" );
			}

		}

		if ( ! $cart_token ) {

			try {

				// create a new Jilt order
				$this->get_api()->create_order( $this->get_cart_data() );

			} catch ( SV_WC_API_Exception $exception ) {

				wc_jilt()->get_logger()->error( "Error communicating with Jilt: {$exception->getMessage()}" );
			}
		}
	}


	/**
	 * When a user intentionally empties their cart, delete the associated Jilt
	 * order
	 *
	 * @since 1.0.0
	 */
	public function cart_emptied() {

		if ( wc_jilt()->get_integration()->is_disabled() ) {
			return;
		}

		$cart_token = WC_Jilt_Session::get_cart_token();

		if ( ! $cart_token ) {
			return;
		}

		WC_Jilt_Session::unset_jilt_order_data();

		try {

			// TODO: need to make sure an order isn't deleted after being placed
			$this->get_api()->delete_order( $cart_token );

		} catch ( SV_WC_API_Exception $exception ) {

			wc_jilt()->get_logger()->error( "Error communicating with Jilt: {$exception->getMessage()}" );
		}
	}


	/**
	 * Get the cart data for updating/creating a Jilt order via the API
	 *
	 * @since 1.0.0
	 * @return array
	 */
	protected function get_cart_data() {

		if ( WC()->cart->is_empty() ) {
			return array();
		}

		$cart_token   = $this->get_cart_token();
		$checkout_url = WC_Jilt_Checkout_Handler::get_checkout_recovery_url( $cart_token );

		$params = array(
			'total_price'       => $this->amount_to_int( $this->get_cart_total() ),
			'subtotal_price'    => $this->get_subtotal_price(),
			'total_tax'         => $this->amount_to_int( WC()->cart->tax_total + WC()->cart->shipping_tax_total ),
			'total_discounts'   => $this->amount_to_int( WC()->cart->discount_cart ),
			'total_shipping'    => $this->amount_to_int( WC()->cart->shipping_total ),
			'requires_shipping' => WC()->cart->needs_shipping(),
			'currency'          => get_woocommerce_currency(),
			'checkout_url'      => $checkout_url,
			'line_items'        => $this->get_cart_product_line_items(),
			'fee_items'         => $this->get_cart_fee_line_items(),
			'client_details'    => $this->get_client_details(),
			'client_session'    => WC_Jilt_Session::get_client_session(),
			'cart_token'        => $cart_token,
		);

		$params = array_merge( $params, $this->get_customer_data() );

		/**
		 * Filters the cart data used for creating or updating a Jilt order
		 * via the API.
		 *
		 * @since 1.0.0
		 *
		 * @param array $params
		 * @param int $order_id optional
		 */
		return apply_filters( 'wc_jilt_order_cart_params', $params, $this );
	}


	/**
	 * Returns the cart token.
	 *
	 * Generates the cart token if necessary and stores it in the session.
	 *
	 * @since 1.4.0
	 *
	 * @return string
	 */
	private function get_cart_token() {

		$cart_token = WC_Jilt_Session::get_cart_token();

		if ( ! $cart_token ) {

			$cart_token = $this->generate_cart_token();

			WC_Jilt_Session::set_jilt_order_data( $cart_token );
		}

		return $cart_token;
	}


	/**
	 * Generates a UUIDv4 cart token.
	 *
	 * @since 1.4.0
	 *
	 * @see https://stackoverflow.com/a/15875555
	 *
	 * @return string
	 */
	private function generate_cart_token() {

		try {
			$data = random_bytes( 16 );

			$data[6] = chr( ord( $data[6] ) & 0x0f | 0x40 ); // set version to 0100
			$data[8] = chr( ord( $data[8] ) & 0x3f | 0x80 ); // set bits 6-7 to 10

			return vsprintf( '%s%s-%s-%s-%s-%s%s%s', str_split( bin2hex( $data ), 4 ) );

		} catch( Exception $e ) {

			// fall back to mt_rand if random_bytes is unavailable
			return sprintf( '%04x%04x-%04x-%04x-%04x-%04x%04x%04x',

				// 32 bits for "time_low"
				mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ),

				// 16 bits for "time_mid"
				mt_rand( 0, 0xffff ),

				// 16 bits for "time_hi_and_version",
				// four most significant bits holds version number 4
				mt_rand( 0, 0x0fff ) | 0x4000,

				// 16 bits, 8 bits for "clk_seq_hi_res",
				// 8 bits for "clk_seq_low",
				// two most significant bits holds zero and one for variant DCE1.1
				mt_rand( 0, 0x3fff ) | 0x8000,

				// 48 bits for "node"
				mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff )
			);
		}

	}


	/**
	 * Get the customer data (email/ID, billing / shipping address) used when
	 * creating/updating an order in Jilt
	 *
	 * @since 1.0.6
	 * @return array
	 */
	protected function get_customer_data() {

		$params = array(
			'billing_address'  => array(),
			'shipping_address' => array(),
		);

		// set the billing/shipping fields from the WC_Customer object
		$params = $this->set_address_fields_from_customer( $params );

		// set customer data from the logged in user, if available. note that this
		// info can be different than the billing/shipping info.
		if ( is_user_logged_in() ) {

			$user = get_user_by( 'id', get_current_user_id() );

			$params['customer'] = array(
				'email'       => $user->user_email,
				'first_name'  => $user->first_name,
				'last_name'   => $user->last_name,
				'customer_id' => $user->ID,
				'admin_url'   => esc_url_raw( add_query_arg( array( 'user_id' => $user->ID ), self_admin_url( 'user-edit.php' ) ) ),
			);

		} elseif ( $this->has_customer_data_from_js_api() ) {

			// set from WC_Customer, if available. currently this should only occur
			// if custom code is using the public JS API to set this data.
			$params['customer'] = array(
				'email'      => SV_WC_Plugin_Compatibility::is_wc_version_gte_3_0() ? WC()->customer->get_billing_email() : WC()->customer->email,
				'first_name' => SV_WC_Plugin_Compatibility::is_wc_version_gte_3_0() ? WC()->customer->get_billing_first_name() : WC()->customer->first_name,
				'last_name'  => SV_WC_Plugin_Compatibility::is_wc_version_gte_3_0() ? WC()->customer->get_billing_last_name() : WC()->customer->last_name,
			);
		}

		return $params;
	}


	/**
	 * Set address fields (billing/shipping) from the customer object in the
	 * session.
	 *
	 * TODO: This is a compatibility method that can be refactored once WC 3.0+ is required. {MR 2017-03-27}
	 *
	 * @since 1.1.0
	 *
	 * @param array $params
	 * @return array
	 */
	private function set_address_fields_from_customer( $params ) {

		foreach ( WC_Jilt_Order::get_jilt_order_address_mapping() as $wc_field => $jilt_field ) {

			if ( SV_WC_Plugin_Compatibility::is_wc_version_gte_3_0() ) {

				$billing_method  = "get_billing_{$wc_field}";
				$shipping_method = "get_shipping_{$wc_field}";

				$billing_value  = WC()->customer->$billing_method();
				$shipping_value = is_callable( array(
					WC()->customer,
					$shipping_method
				) ) ? WC()->customer->$shipping_method() : null;

			} else {

				$billing_property  = $wc_field;
				$shipping_property = "shipping_{$wc_field}";

				$billing_value  = WC()->customer->$billing_property;
				$shipping_value = WC()->customer->$shipping_property;
			}

			$params['billing_address'][ $jilt_field ]  = ( $billing_value !== '' ) ? $billing_value : null;
			$params['shipping_address'][ $jilt_field ] = ( $shipping_value !== '' ) ? $shipping_value : null;
		}

		return $params;
	}


	/**
	 * Return true if customer data was set from the JS API.
	 *
	 * TODO: This is a compatibility method that can be refactored once WC 3.0+ is required. {MR 2017-03-27}
	 *
	 * @since 1.1.0
	 * @return bool
	 */
	private function has_customer_data_from_js_api() {

		if ( SV_WC_Plugin_Compatibility::is_wc_version_gte_3_0() ) {
			return WC()->customer->get_first_name() || WC()->customer->get_last_name() || WC()->customer->get_billing_email();
		} else {
			WC()->customer->first_name || WC()->customer->last_name || WC()->customer->email;
		}
	}


	/**
	 * Get the cart subtotal, in pennies
	 *
	 * @since 1.3.0
	 * @return int the subtotal price, maybe inclusive of taxes, in pennies
	 */
	public function get_subtotal_price() {

		if ( 'excl' === get_option( 'woocommerce_tax_display_cart' ) ) {
			$subtotal = WC()->cart->subtotal_ex_tax;
		} else {
			$subtotal = WC()->cart->subtotal;
		}

		return $this->amount_to_int( $subtotal );
	}


	/**
	 * Map WC cart items to Jilt line items
	 *
	 * @since 1.0.0
	 * @return array Mapped line items
	 */
	private function get_cart_product_line_items() {

		$line_items = array();

		// products
		foreach ( WC()->cart->get_cart() as $item_key => $item ) {

			$product = $item['data'];
			if ( ! $product instanceof WC_Product ) {
				continue;
			}

			// prepare main line item params
			$line_item = array(
				'title'      => html_entity_decode( $product->get_title() ),
				'product_id' => $item['product_id'],
				'quantity'   => $item['quantity'],
				'sku'        => $product->get_sku(),
				'url'        => get_the_permalink( $item['product_id'] ),
				'image_url'  => WC_Jilt_Product::get_product_image_url( $product ),
				'key'        => $item_key,
				'price'      => $this->get_item_price( $product ),
				'tax_lines'  => $this->get_tax_lines( $item ),
				'properties' => array(),
			);

			// add variation data
			if ( ! empty( $item['variation_id'] ) ) {

				$line_item['variant_id'] = $item['variation_id'];
				$line_item['variation']  = WC_Jilt_Product::get_variation_data( $item );
			}

			/**
			 * Filter cart item params used for creating/updating a Jilt order
			 * via the API
			 *
			 * @since 1.0.0
			 *
			 * @param array $line_item Jilt line item data
			 * @param array $item WC line item data
			 * @param string $item_key WC cart key for item
			 */
			$line_items[] = apply_filters( 'wc_jilt_order_cart_item_params', $line_item, $item, $item_key );
		}

		return $line_items;
	}


	/**
	 * Get the tax lines, if any, for this item
	 *
	 * @since 1.3.0
	 *
	 * @param array Item associative array
	 * @return array of tax lines, e.g. [ [ 'amount' => 135 ] ]
	 */
	private function get_tax_lines( $item ) {

		// a simplistic implementation for now, but if WC identifies the actual
		// taxes a la Shopify, we can make this more comprehensive
		return array(
			array(
				'amount' => $this->amount_to_int( $item['line_tax'] ),
			),
		);
	}


	/**
	 * Map WC cart fee line items to Jilt fee items
	 *
	 * @since 1.1.0
	 * @return array Mapped fee items
	 */
	private function get_cart_fee_line_items() {

		$fee_items = array();

		// fees
		if ( $fees = WC()->cart->get_fees() ) {
			foreach ( $fees as $fee ) {

				$fee_item = array(
					'title'  => html_entity_decode( $fee->name ),
					'key'    => $fee->id,
					'amount' => $this->amount_to_int( $fee->amount ),
				);

				/**
				 * Filter cart fee item params used for creating/updating a Jilt order
				 * via the API
				 *
				 * @since 1.1.0
				 *
				 * @param array $fee_item Jilt fee item data
				 * @param \stdClass $fee WC fee object
				 */
				$fee_items[] = apply_filters( 'wc_jilt_order_cart_fee_params', $fee_item, $fee );
			}
		}

		return $fee_items;
	}


	/**
	 * Get any client details for the cart
	 *
	 * @since 1.2.0
	 * @return array associative array of client details (if available)
	 */
	private function get_client_details() {

		$client_details = array();

		if ( $browser_ip = WC_Geolocation::get_ip_address() ) {
			$client_details['browser_ip'] = $browser_ip;
		}
		if ( ! empty( $_SERVER['HTTP_ACCEPT_LANGUAGE'] ) ) {
			$client_details['accept_language'] = $_SERVER['HTTP_ACCEPT_LANGUAGE'];
		}
		if ( ! empty( $_SERVER['HTTP_USER_AGENT'] ) ) {
			$client_details['user_agent'] = $_SERVER['HTTP_USER_AGENT'];
		}

		return $client_details;
	}


	/**
	 * Return the cart total. WC does not set the cart total unless on the
	 * cart or checkout pages - see WC_Cart:calculate_totals() so on other
	 * pages it's calculated manually.
	 *
	 * @since 1.0.6
	 * @return double
	 */
	protected function get_cart_total() {

		if ( is_checkout() || is_cart() || defined( 'WOOCOMMERCE_CHECKOUT' ) || defined( 'WOOCOMMERCE_CART' ) ) {

			// on cart/checkout, total is calculated
			return WC()->cart->total;

		} else {

			// likely on product page, etc. total is not calculated but tax/shipping may be available
			return WC()->cart->subtotal_ex_tax + WC()->cart->tax_total + WC()->cart->shipping_tax_total + WC()->cart->shipping_total;
		}
	}


	/**
	 * Get the product price, either inclusive or exclusive of tax, depending
	 * on the WooCommerce "display cart prices inclusive/exclusive of taxes"
	 * setting.
	 *
	 * Note: this is adapted from WC_Cart::get_product_price()
	 *
	 * @since 1.3.0
	 *
	 * @param WC_Product $product the product
	 * @return int the product price in pennies
	 */
	private function get_item_price( $product ) {

		if ( 'excl' == get_option( 'woocommerce_tax_display_cart' ) ) {
			$price = SV_WC_Product_Compatibility::wc_get_price_excluding_tax( $product );
		} else {
			$price = SV_WC_Product_Compatibility::wc_get_price_including_tax( $product );
		}

		return $this->amount_to_int( $price );
	}


	/**
	 * Convert a price/total to the lowest currency unit (e.g. cents)
	 *
	 * @since 1.0.6
	 *
	 * @param string|float $number
	 *
	 * @return int
	 */
	private function amount_to_int( $number ) {

		if ( is_string( $number ) ) {
			$number = (float) $number;
		}

		return round( $number * 100, 0 );
	}


	/**
	 * Helper method to improve the readability of methods calling the API
	 *
	 * @since 1.0.0
	 * @return \WC_Jilt_API instance
	 */
	private function get_api() {
		return wc_jilt()->get_integration()->get_api();
	}


	/**
	 * Checks whether we have the customer's email or not.
	 *
	 * @since 1.4.0
	 *
	 * @return bool
	 */
	private function has_customer_email() {

		if ( SV_WC_Plugin_Compatibility::is_wc_version_gte_3_0() ) {

			return is_callable( array( WC()->customer, 'get_billing_email' ) ) && WC()->customer->get_billing_email();

		} else {

			return is_object( WC()->customer ) && ! empty( WC()->customer->email );
		}
	}


	/**
	 * Checks whether we should try to capture customer's email on add to cart.
	 *
	 * @since 1.4.0
	 *
	 * @return bool
	 */
	private function should_capture_email_on_add_to_cart() {
		return wc_jilt()->get_integration()->capture_email_on_add_to_cart()
		       && ! is_user_logged_in()
		       && ! WC()->session->get( 'jilt_opt_out_add_to_cart_email_capture' )
		       && ! $this->has_customer_email();
	}


}
